if (MCE!=undefined && MCE.applicationName=='Converter') {
MCE.prefs={
	currencyControlsInitialized: false,

	// Describes the schema
	pref_list:{
		pref_24h:'boolean',
		pref_fullpage_icon:'boolean',
		pref_fullpage_menu:'boolean',
		pref_selection_menu:'boolean',
		pref_metric:'boolean',
		pref_celsius:'boolean',
		pref_in_fract:'boolean',
		pref_custom_single:'boolean',
		pref_currency_enabled:'boolean',
		pref_myCurrency:'string',
		pref_custom_convert_selection:'boolean',
		pref_thou_sep:'string',
		pref_dec_sep:'string'
	},
	
	// Describes the defaults
	pref_defaults:{
		pref_24h:true,
		pref_fullpage_icon:true,
		pref_fullpage_menu:false,
		pref_selection_menu:true,
		pref_metric:true,
		pref_celsius:true,
		pref_in_fract:true,
		pref_custom_single:true,
		pref_currency_enabled:true,
		pref_myCurrency:'',
		pref_custom_convert_selection:true,
		pref_thou_sep:',',
		pref_dec_sep:'.'
	},
	
	// Contains the cached prefs
	prefs:{
	},

	// Have we cached the prefs?
	prefs_cached:false,
	
	// Our very own nsIPrefBranch object; it gets destroyed after each operation
	pO:false,
	
	// public; direct use encouraged
	getPref:function(pref)
	{
		this.buildCache();
		return this.prefs[pref];
	},
	
	// public; direct use encouraged
	setPref:function(pref,val)
	{
		if (!this._writePref(pref,val)) {
			return false;
		}
		this.prefs[pref]=val;
		return true;
	},
	
	// public; direct use discouraged
	buildCache:function()
	{
		if (this.prefs_cached) {
			return true;
		}
		var prefs=this.pO;
		for(var pref in this.pref_list) {
			if (prefs.getPrefType(pref)==prefs.PREF_INVALID) {
				this.setPref(pref,this.pref_defaults[pref]);
			}
			this.prefs[pref]=this._readPref(pref);
		}
		return true;
	},
	
	// public; direct use discouraged
	clearCache:function()
	{
		this.prefs_cached=false;
	},

	init:function()
	{
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService);
		prefs = prefs.getBranch("extensions.converter.preferences.");
		this.pO=prefs;
	},
	
	// private
	_readPref:function(pref)
	{
		if(this.pref_list[pref]=='boolean') {
			return this.pO.getBoolPref(pref);
		}
		if (this.pref_list[pref]=='string') {
			return this.pO.getCharPref(pref);
		}
		// to be extended
		return null;
	},
	
	// private
	_writePref:function(pref,val)
	{
		// Too lazy to check whether Moz prefs match exactly on JS types, so
		// I'll check manually for now.
		if(this.pref_list[pref]=='boolean') {
			if(typeof(val)!='boolean') {
				return false;
			}
			this.pO.setBoolPref(pref,val);
			return true;
		}
		if (this.pref_list[pref]=='string') {
			this.pO.setCharPref(pref,val);
			return true;
		}
		// to be extended
		return null;
	},

	prefsOnLoad:function()
	{
		// Currency
		this.initCurrencyControls();
		
		return true;
	},

	initCurrencyControls:function()
	{
		var list=document.getElementById("pref_myCurrency");
		if (this.currencyControlsInitialized) {
			if (list.value && list.getItemAtIndex(0).value=='') {
				list.removeItemAt(0);
			}
			return true;
		}

		this.currencyControlsInitialized=true;

		var listIndex=MCE.util.inArray(this.getPref('pref_myCurrency'),MCE.currency.currencies);
		if (listIndex==-1) {
			listIndex=0;
			list.appendItem("Please select one in order to use this feature",'');
		}
		var cc; // Currency code
		var cn; // Currency name
		for(var i=0;i<MCE.currency.currencies.length;i++) {
			cc=MCE.currency.currencies[i];
			if (cc in MCE.currency.currency_names) {
				cn=' ('+MCE.currency.currency_names[cc]+')';
			} else {
				cn='';
			}
			list.appendItem(
				cc, /* label */
				cc, /* value */
				cn /* description */
			);	
		}
		list.selectedIndex=listIndex;
		window.sizeToContent();
		return true;
	},

	close_ok:function()
	{
		if (
			document.getElementById("pref_currency_enabled").checked &&
			!document.getElementById("pref_myCurrency").value
		) {
			// I'd be doing this instead on onbeforeaccept if it weren't
			// for Bug#488556
			//document.getElementById("pref_currency_enabled").checked=false;
			this.setPref("pref_currency_enabled",false);
			alert(
				"Currency conversions have been disabled.\n"+
				"You can enable them at any time in the options dialog."
			);
			// See Bug#474527 -- returning false here on onbeforeaccept
			// acts as cancel.
			// return false;
		}
		// All is well, let's update the status bar icon in all windows
		var wm =
			Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
		var enumerator = wm.getEnumerator('navigator:browser');
		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			var statIcon=win.document.getElementById('converter_status');
			if (statIcon) {
				statIcon.hidden = !this.getPref('pref_fullpage_icon');
			}
		}

		return true;
	},

	close_cancel:function()
	{
		if (
			this.getPref("pref_currency_enabled") &&
			!this.getPref("pref_myCurrency")
		) {
			document.getElementById("pref_currency_enabled").checked=false;
			this.setPref("pref_currency_enabled",false);
			
			alert(
				"Currency conversions have been disabled.\n"+
				"You can enable them at any time in the options dialog."
			);
		}
		return true;
	}

}

MCE.prefs.init();

}
